const filename = require('./filename')

const winston = require('winston')
const util = require('util')
const _ = require('lodash')

const { createLogger, format, transports } = winston
const { combine, timestamp, printf } = format
const { LEVEL } = require('triple-beam')

const level = process.env.loglevel || process.env.LOGLEVEL || 'info'

let logFileName = false
switch (level) {
case 'silly':
case 'debug':
  logFileName = true
  break
}

const extendedInfo = (message) => {
  const padLength = 30

  if (logFileName) {
    return `${_.padEnd(message, padLength)} (${filename()})`
  }
  return message
}

const procID = process.pid
const datePad = (str) => {
  return _.padStart(str, 2, '0')
}
const dateString = () => {
  const theDate = new Date()
  const fullYear = theDate.getFullYear().toString()
  const smallYear = `${fullYear[2]}${fullYear[3]}`
  const month = datePad(theDate.getMonth() + 1)
  const day = datePad(theDate.getDate())
  const hours = datePad(theDate.getHours())
  const minutes = datePad(theDate.getMinutes())
  const seconds = datePad(theDate.getSeconds())
  return `${smallYear}/${month}/${day} ${hours}:${minutes}:${seconds}`
}

const ferrariFormat = printf(info => {
  if (info.level === 'silly') {
    info.level = 'develop'
  }
  if (info.level === 'warn') {
    info.level = 'warning'
  }
  if (typeof info.message !== 'string') {
    info.message = util.inspect(info.message, false, 10)
  }
  // (3544) 19/08/13 17:19:05 - ERROR: IfHndFfoe::SetFileList(): no file found for document 0.
  return `(${procID}) ${dateString()} - ${info.level.toUpperCase()}:${_.repeat(' ', 7 - info.level.length)} ${extendedInfo(info.message)}`
})

const logger = createLogger({
  format: combine(
    ferrariFormat
  ),
  exceptionHandlers: [
    new winston.transports.Console()
  ],
  exitOnError: false
})

logger.add(new transports.Console())

logger.level = level

module.exports = logger
